require("Atlas-Addon-Helpers.AtlasHelpers");
luanet.load_assembly("log4net");

local types = {}
types["log4net.LogManager"] = luanet.import_type("log4net.LogManager");
local log = types["log4net.LogManager"].GetLogger("AtlasSystems.Addons.RoutingNotification");

local Settings = {};
Settings.NVTGC = GetSetting("NVTGC");
Settings.MonitorQueues = GetSetting("MonitorQueues");
Settings.EmailName = GetSetting("EmailName");
Settings.RouteQueue = GetSetting("RouteQueue");


function Init()
    log:Debug("Initializing Routing & Notification addon.");
    InitializeSettings();
    RegisterSystemEventHandler("SystemTimerElapsed", "CheckMonitoredQueues");
end

function InitializeSettings()
    local nvtgcs = {};
    for _, nvtgc in ipairs(AtlasHelpers.StringSplit(",", Settings.NVTGC)) do
        nvtgc = AtlasHelpers.Trim(nvtgc);
        if nvtgc ~= "" then
            table.insert(nvtgcs, nvtgc);
        end
    end
    Settings.NVTGC = nvtgcs;
    local queues = {};
    for _, queue in ipairs(AtlasHelpers.StringSplit(",", Settings.MonitorQueues)) do
        queue = AtlasHelpers.Trim(queue);
        if queue ~= "" then
            table.insert(queues, queue);
        end
    end
    Settings.MonitorQueues = queues;

end

function CheckMonitoredQueues()
    for _, queue in ipairs(Settings.MonitorQueues) do
        local queueName = AtlasHelpers.Trim(queue);
        log:DebugFormat("Processing requests in status {0}", queueName);
        ProcessDataContexts("TransactionStatus", queueName, "RouteAndNotify");
    end
end

function RouteAndNotify()
    local tn = tonumber(GetFieldValue("Transaction","TransactionNumber"));
    local nvtgc = GetFieldValue("User", "NVTGC");

	if (not ShouldHandleRequest(nvtgc)) then
        log:InfoFormat("Skipping processing transaction {0}. NVTGC ({1}) is not configured for processing.", tn, nvtgc);
		return;
    end
    
    if (Settings.RouteQueue ~= nil and Settings.RouteQueue ~= "") then
        log:DebugFormat("Routing transaction {0} to {1}", tn, Settings.RouteQueue);
        ExecuteCommand("Route", {tn, Settings.RouteQueue});

        if (Settings.EmailName ~= nil and Settings.EmailName ~= '') then
            log:DebugFormat("Sending email template {0} for transaction {1}", Settings.EmailName, tn);
            ExecuteCommand("SendTransactionNotification", {tn, Settings.EmailName});
        end
    end
end

function ShouldHandleRequest(userNvtgc)
	if (#Settings.NVTGC > 0) then
        log:Debug("Checking if the request should be processed for  NVTGC (" .. userNvtgc .. ")");

        for index, nvtgc in ipairs(Settings.NVTGC) do
            if (nvtgc:lower() == userNvtgc:lower()) then
                return true;
            end
		end

		return false;
	else
		--Handle all requests because NVTGC is not configured
		return true;
	end
end